/* ------------------------------------------------------------------ */
/*      item            : LogMessage.hxx
        generated by    : Autobuild (abuild)
        date            : Fri 07 Mar 2025
        category        : header file
        description     : DUECA Communication Object (DCO)
                          automatically generated by dueca-codegen
        codegen version : 111
        language        : C++
        item            : LogMessage.dco
        made by         : Rene' van Paassen
        date            : ??
        description     : Logging information transfer
        copyright       : (c) 2016-2022 TUDelft-AE-C&S
                        : (c) 2022 René van Paassen
        license         : EUPL-1.2
*/

#pragma once
#if !defined(__DCO_NOPACK)
namespace dueca {
class AmorphStore;
class AmorphReStore;
struct DataWriterArraySize;
};
#endif

#if !defined(__DCO_STANDALONE)
namespace dueca {
struct CommObjectDataTable;
};

#include <gencodegen.h>
#if GENCODEGEN != 111
#error "Generated LogMessage.hxx too old, please clean with 'make mrproper'"
#endif
#include <dueca/CommObjectTraits.hxx>
#endif
#include <iostream>

#ifndef NESTED_DCO
#define NESTED_DCO
#define MASTER_DCO_LogMessage
#endif
#include <LogTime.hxx>
#include <ActivityContext.hxx>
#include <LogString.hxx>
#ifdef MASTER_DCO_LogMessage
#undef MASTER_DCO_LogMessage
#undef NESTED_DCO
#endif

namespace dueca {
 
/** This event represents a log message from somewhere in the system
    */
struct LogMessage
{
  /** typedef for internal reference */
  typedef LogMessage __ThisDCOType__;

public:
  /** The name of this class. */
  static const char* const classname;


  /** logpoints are unique ID's for logging "possibilities". Each
      logpoint is associated with a certain node, a certain file,
      and a line number in that file. This message is generated by
      that logpoint */
  uint32_t logpoint;

  /** the number of times a message has been generated at this
      logpoint is also recorded */
  uint32_t count;

  /** the time at which this message was generated. */
  LogTime time;

  /** the activity context, consisting of a reference to the activity
      manager, node id and the activity number */
  ActivityContext context;

  /** the error message is a string, maximum size 236 bytes
      this size keeps the LogMessage to 256 bytes total, and thus neatly
      packed in a memory arena. */
  LogString message;

public:
  /** a "magic" number, hashed out of the class definition,
      that will be used to check consistency of the sent objects
      across the dueca nodes. */
  static const uint32_t magic_check_number;

  /** default constructor. */
  LogMessage();

  /** Constructor with arguments */
  LogMessage(
        const uint32_t& logpoint,
        const uint32_t& count,
        const LogTime& time,
        const ActivityContext& context,
        const LogString& message);

  /** copy constructor. */
  LogMessage(const LogMessage& o);

#if !defined(__DCO_NOPACK)
  /** constructor to restore an LogMessage from amorphous storage. */
  LogMessage(::dueca::AmorphReStore& r);
#endif

  /** destructor. */
  ~LogMessage();

#if !defined(__DCO_STANDALONE)
  /** new operator "new", which places objects not on a
      heap, but in one of the memory arenas. This to speed up
      memory management. */
  static void* operator new(size_t size);

  /** accompanying placement operator "new" */
  static void* operator new(size_t size, void* mem);

  /** new operator "delete", to go with the new version
      of operator new. */
  static void operator delete(void* p);

  /** placement "new", needed for stl. */
  inline static void* operator new(size_t size, LogMessage*& o)
  { return reinterpret_cast<void*>(o); }
#endif

#if !defined(__DCO_NOPACK)
  /** packs the LogMessage into amorphous storage. */
  void packData(::dueca::AmorphStore& s) const;

  /** packs the LogMessage into amorphous storage.
      only differences with a previous object are packed. */
  void packDataDiff(::dueca::AmorphStore& s, const LogMessage& ref) const;

  /** unpacks the LogMessage from an amorphous storage. */
  void unPackData(::dueca::AmorphReStore& s);

  /** unpacks the differences for LogMessage
      from an amorphous storage. */
  void unPackDataDiff(::dueca::AmorphReStore& s);
#endif

  /** Test for equality. */
  bool operator == (const LogMessage& o) const;

  /** Test for inequality. */
  inline bool operator != (const LogMessage& o) const
  { return !(*this == o); }

  /** Assignment operator. */
  LogMessage& operator=(const LogMessage& o);

  /** prints the LogMessage to a stream. */
  std::ostream & print(std::ostream& s) const;

// Additional custom code
#include "LogMessageExtra.hxx"
};

} 

#if !defined(__DCO_NOPACK)
/** pack the object into amorphous storage. */
inline void packData(::dueca::AmorphStore& s,
                     const dueca::LogMessage& o)
{ o.packData(s); }

/** pack the differences between this object and another
    into amorphous storage. */
inline void packDataDiff(dueca::AmorphStore& s,
                         const dueca::LogMessage& o,
                         const dueca::LogMessage& ref)
{ o.packDataDiff(s, ref); }

/** unpack the object from amorphous storage. */
inline void unPackData(::dueca::AmorphReStore& s,
                       dueca::LogMessage& o)
{ o.unPackData(s); }

/** unpack the differences to this object from storage. */
inline void unPackDataDiff(dueca::AmorphReStore& s,
                           dueca::LogMessage& o)
{ o.unPackDataDiff(s); }
#endif

namespace std {
/** print to a stream. */
inline std::ostream &
operator << (std::ostream& s, const dueca::LogMessage& o)
{ return o.print(s); }
};

// specialization of classname
template<>
const char* dueca::getclassname<dueca::LogMessage>();
template <>
inline const char* dueca::getclassname<const dueca::LogMessage>()
{ return dueca::getclassname<dueca::LogMessage>(); }

#if !defined(__DCO_STANDALONE)
namespace dueca {
/** Template specialization, defines a trait that is needed if
    LogMessage is ever used inside other dco objects. */
template <>
struct dco_nested<dueca::LogMessage> : public dco_isnested { };
};

#endif


